@@ -3,7 +3,7 @@
 ###use Data::Dumper ; print Dumper( $XML->tree ) ;
 
 use Test;
-BEGIN { plan tests => 84 } ;
+BEGIN { plan tests => 96 } ;
 use XML::Smart ;
 
 no warnings ;
@@ -23,6 +23,127 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
 </hosts>
 `;
 
+#########################
+{
+
+  my $XML = XML::Smart->new( q`
+  <html>
+    <head>
+      <title>Blah blah</title>
+    </head>
+    <body>
+      <form>
+        <input id="0"/>        
+        <br/>
+        <input id="2"/>
+        <br/>            
+      </form>
+    </body>
+    <null/>
+  </html>
+  ` );
+  
+  my $data = $XML->data( noheader => 1 ) ;
+  $data =~ s/\s+/ /gs ;
+  
+  ok($data , q`<html> <head> <title>Blah blah</title> </head> <body> <form> <input id="0"/> <br/> <input id="2"/> <br/> </form> </body> <null/> </html> `) ;
+  
+  my @order = $XML->{html}{body}{form}->order ;
+  ok(join(" ", @order) eq 'input br input br') ;
+  
+  $XML->{html}{body}{form}->set_order( qw(br input input br) ) ;
+  @order = $XML->{html}{body}{form}->order ;
+  ok(join(" ", @order) eq 'br input input br') ;
+
+  my $data = $XML->data( noheader => 1 ) ;
+  $data =~ s/\s+/ /gs ;
+  
+  ok($data , q`<html> <head> <title>Blah blah</title> </head> <body> <form> <br/> <input id="0"/> <input id="2"/> <br/> </form> </body> <null/> </html> `) ;
+
+}
+#########################
+{
+
+  my $XML = XML::Smart->new(q`
+<root>
+content0
+<tag1 arg1="123">
+  <sub arg="1">sub_content</sub>
+</tag1>
+content1
+<tag2 arg1="123"/>
+content2
+</root>
+  ` , 'XML::Smart::Parser') ;
+  
+  my $data = $XML->data(noheader => 1) ;
+  
+  ok($data , q`<root>
+content0
+<tag1 arg1="123">
+    <sub arg="1">sub_content</sub></tag1>
+content1
+<tag2 arg1="123"/>
+content2
+</root>
+
+`) ;
+  
+  ok( tied $XML->{root}->pointer->{CONTENT} ) ;
+  
+  my $cont = $XML->{root}->{CONTENT} ;
+  
+  ok($cont , q`
+content0
+
+content1
+
+content2
+`) ;
+  
+  my $cont_ = $XML->{root}->content ;
+
+  ok($cont_ , q`
+content0
+
+content1
+
+content2
+`) ;
+  
+  $XML->{root}->content(1,"set1") ;
+  
+  my @cont = $XML->{root}->content ;
+  
+  ok($cont[0] , "\ncontent0\n") ;
+  ok($cont[1] , "set1") ;
+  ok($cont[2] , "\ncontent2\n") ;
+  
+  $XML->{root}->{CONTENT} = 123 ;
+  
+  my $cont_2 = $XML->{root}->content ;
+  
+  ok($cont_2 , 123) ;
+  
+  ok( !tied $XML->{root}->pointer->{CONTENT} ) ;
+  
+  ok( !tied $XML->{root}{tag1}{sub}->pointer->{CONTENT} ) ;
+  
+  my $sub_cont = $XML->{root}{tag1}{sub}->{CONTENT} ;
+  
+  ok($sub_cont , 'sub_content') ;
+  
+  my $data = $XML->data(noheader => 1) ;
+  
+  ok($data , q`<root>
+  <tag1 arg1="123">
+    <sub arg="1">sub_content</sub>
+  </tag1>
+  <tag2 arg1="123"/>123</root>
+
+`) ;
+  
+}
 #########################
 {
 
@@ -124,13 +245,14 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
   
   my $XML = XML::Smart->new() ;
   
-  $XML->{sub}{sub2} = $XML0 ;
-  push(@{$XML->{sub}{sub2}} , $XML1 ) ;
+  $XML->{sub}{sub2} = $XML0->tree ;
+  push(@{$XML->{sub}{sub2}} , $XML1->tree ) ;
   
   my $data = $XML->data(noheader => 1) ;
   
   $data =~ s/\s//gs ;
   ok($data,'<sub><sub2><root><foo1name="x"/></root></sub2><sub2><root><foo2name="y"/></root></sub2></sub>') ;
+
 }
 #########################
 {
@@ -251,6 +373,7 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
     
   my $dataok = q`<serveros="Linux"type="mandrake"version="8.9"><address>192.168.3.201</address><address>192.168.3.202</address></server>`;
   ok($data,$dataok) ;
+
 }
 #########################
 {
@@ -284,6 +407,7 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
   
   my $dataok = q`<HOSTS><SERVEROS="linux"TYPE="redhat"VERSION="8.0"><ADDRESS>192.168.0.1</ADDRESS><ADDRESS>192.168.0.2</ADDRESS></SERVER><SERVEROS="linux"TYPE="suse"VERSION="7.0"><ADDRESS>192.168.1.10</ADDRESS><ADDRESS>192.168.1.20</ADDRESS></SERVER><SERVERADDRESS="192.168.2.100"OS="linux"TYPE="conectiva"VERSION="9.0"/><SERVERADDRESS="192.168.3.30"OS="bsd"TYPE="freebsd"VERSION="9.0"/></HOSTS>`;
   ok($data,$dataok) ;
+  
 }
 #########################
 {
@@ -320,7 +444,7 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
   
   ok($data,$dataok) ;
                        
-  $XML->{hosts}[1]{server}[1] = {
+  $XML->{hosts}[1]{server}[0] = {
   os => 'LX'  ,
   type => 'red'  ,
   ver => 123 ,
@@ -332,15 +456,14 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
   $dataok = q`<root><hosts><serveros="lx"type="red"ver="123"/></hosts><hosts><serveros="LX"type="red"ver="123"/></hosts></root>`;
   
   ok($data,$dataok) ;
-  
-}
 
+}
 #########################
 {
 
   my $XML = XML::Smart->new('' , 'XML::Smart::Parser') ;
                           
-  $XML->{hosts}[1]{server}[1] = {
+  $XML->{hosts}[1]{server}[0] = {
   os => 'LX'  ,
   type => 'red'  ,
   ver => 123 ,
@@ -352,7 +475,7 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
   $dataok = q`<hosts><serveros="LX"type="red"ver="123"/></hosts>`;
   
   ok($data,$dataok) ;
-  
+
 }
 #########################
 {
@@ -366,12 +489,15 @@ my $DATA = q`<?xml version="1.0" encoding="iso-8859-1"?>
   } ;
 
   push( @{$XML->{hosts}} , {XXXXXX => 1}) ;
-  unshift( @{$XML->{hosts}{x}}  , $srv) ;
+  
+  unshift( @{$XML->{hosts}}  , $srv) ;
+  
+  push( @{$XML->{hosts}{more}}  , {YYYY => 1}) ;
   
   my $data = $XML->data(noheader => 1) ;
   $data =~ s/\s//gs ;
   
-  $dataok = q`<root><hostsos="lx"type="red"ver="123"/><hostsXXXXXX="1"/></root>` ;
+  $dataok = q`<root><hostsos="lx"type="red"ver="123"><moreYYYY="1"/></hosts><hostsXXXXXX="1"/></root>` ;
   
   ok($data,$dataok) ;
 
